#!/usr/bin/env perl
use strict;
use warnings;

package Term;
use base qw/Class::Accessor::Lvalue::Fast/;

__PACKAGE__->mk_accessors(qw/term tf/);

package main;

use HTML::ExtractContent;
use LWP::UserAgent;
use Text::MeCab;

my $url = shift or die "usage: $0 <url>\n";

my $text = url2text($url);
my @terms = text2terms($text);

for my $t (@terms) {
    printf "%2d\t%s\n", $t->tf, $t->term;
}

sub url2text {
    my $url = shift;

    my $ua  = LWP::UserAgent->new;
    my $res = $ua->get($url);

    if ($res->is_error) {
        die $res->status_line;
    }

    my $html = $res->decoded_content;
    my $ex = HTML::ExtractContent->new;
    $ex->extract($html);
    return $ex->as_text;
}

sub text2terms {
    my $text = shift;
    my $mecab = Text::MeCab->new;
    my %tf;
    for (my $node = $mecab->parse($text); $node; $node = $node->next) {
        next if $node->feature !~ m/^名詞/;
        next if $node->feature !~ m/(一般|固有名詞)/;
        next if $node->feature =~ m/(非自立|代名詞|接尾)/;
        warn sprintf "%s\t%s", $node->surface, $node->feature;
        $tf{$node->surface}++;
    }

    return map {
        my $t = Term->new;
        $t->term = $_;
        $t->tf   = $tf{$_};
        $t;
    } keys %tf;
}
