#!/usr/bin/env perl
use strict;
use warnings;

package Term;
use base qw/Class::Accessor::Lvalue::Fast/;

use URI;
use Encode;

__PACKAGE__->mk_accessors(qw/term tf/);

sub df {
    my $self = shift;

    my $yahoo = URI->new('http://search.yahooapis.jp/WebSearchService/V1/webSearch');
    $yahoo->query_form(
        appid   => "...", #アプリケーションIDを指定
        query   => encode_utf8($self->term),
        results => 1,
    );

    my $ua  = LWP::UserAgent->new;
    my $res = $ua->get($yahoo);

    if ($res->is_error) {
        die $res->status_line;
    }

    my ($df) = ($res->content =~ /totalResultsAvailable="(\d+)"/);
    return $df;
}

sub idf {
    my $self = shift;
    my $df = $self->df or return 0;
    return log(19200000000/$df);
}

sub tfidf {
    my $self = shift;
    return $self->tf * $self->idf;
}

package main;

use HTML::ExtractContent;
use LWP::UserAgent;
use XMLRPC::Lite;
use List::MoreUtils qw/uniq/;
use Regexp::Trie;
use Encode;

my $url = shift or die "usage: $0 <url>\n";
my $text = url2text($url);
my @terms = text2terms($text);

for my $t (@terms) {
    printf "%.1f\t%s\n", $t->tfidf, encode_utf8($t->term);
}

sub url2text {
    my $url = shift;

    my $ua  = LWP::UserAgent->new;
    my $res = $ua->get($url);

    if ($res->is_error) {
        die $res->status_line;
    }

    my $html = $res->decoded_content;
    my $ex = HTML::ExtractContent->new;
    $ex->extract($html);
    return $ex->as_text;
}

sub text2terms {
    my $text = shift;
    my @terms = hatena_keyword($text);

    ## Regexp::Trie を使って tf を求める
    my $rt = Regexp::Trie->new;
    $rt->add($_) for @terms;
    my $re = $rt->regexp;

    my %tf;
    while ($text =~ /($re)/gi) {
        $tf{$1}++;
    }

    return map {
        my $t = Term->new;
        $t->term = $_;
        $t->tf   = $tf{$_};
        $t;
    } keys %tf;
}

sub hatena_keyword {
    my $body = shift;

    my $rpc = XMLRPC::Lite->new;
    $rpc->proxy('http://d.hatena.ne.jp/xmlrpc');
    my $res = $rpc->call(
        'hatena.setKeywordLink',
        {
            body  => XMLRPC::Data->type('string', $body),
            mode  => 'lite',
            score => 0,
        }
    );

    if (my $fault = $res->fault) {
        for (keys %{$fault}) {
            warn sprintf "%s => %s", $_, $fault->{$_};
        }
        die;
    }

    return uniq map { $_->{word} } @{$res->result->{wordlist}};
}
