#!/usr/bin/env perl
use strict;
use warnings;

package Term;
use base qw/Class::Accessor::Lvalue::Fast/;

use URI;
use Encode;

__PACKAGE__->mk_accessors(qw/term tf df/);

sub idf {
    my $self = shift;
    my $df = $self->df or return 0;
    return log(19200000000/$df);
}

sub tfidf {
    my $self = shift;
    return $self->tf * $self->idf;
}

package main;

use HTML::ExtractContent;
use LWP::UserAgent;
use XMLRPC::Lite;
use List::MoreUtils qw/uniq/;
use Regexp::Trie;
use Encode;

my $url = shift or die "usage: $0 <url>\n";
my $text = url2text($url);
my @terms = text2terms($text);

for my $t (@terms) {
    printf "%.1f\t%s\n", $t->tfidf, encode_utf8($t->term);
}

sub url2text {
    my $url = shift;

    my $ua  = LWP::UserAgent->new;
    my $res = $ua->get($url);

    if ($res->is_error) {
        die $res->status_line;
    }

    my $html = $res->decoded_content;
    my $ex = HTML::ExtractContent->new;
    $ex->extract($html);
    return $ex->as_text;
}

sub text2terms {
    my $text = shift;
    my @terms = hatena_keyword($text);

    ## Regexp::Trie を使って tf を求める
    my $rt    = Regexp::Trie->new;
    my %terms;
    for (@terms) {
        $rt->add($_->term);
        $terms{$_->term} = $_;
    }
    my $re = $rt->regexp;

    while ($text =~ /($re)/gi) {
        if ($terms{$1}) {
            $terms{$1}->tf++;
        }
    }

    return @terms;
}

sub hatena_keyword {
    my $body = shift;

    my $rpc = XMLRPC::Lite->new;
    $rpc->proxy('http://d.hatena.ne.jp/xmlrpc');
    my $res = $rpc->call(
        'hatena.setKeywordLink',
        {
            body  => XMLRPC::Data->type('string', $body),
            mode  => 'lite',
            score => 0,
        }
    );

    if (my $fault = $res->fault) {
        for (keys %{$fault}) {
            warn sprintf "%s => %s", $_, $fault->{$_};
        }
        die;
    }

    my %seen;
    my @terms;
    for (@{$res->result->{wordlist}}) {
        next if $seen{$_->{word}}++; # uniq

        my $t = Term->new;
        $t->term = $_->{word};
        $t->tf   = 0;
        $t->df   = $_->{refcount}; # refcount == df

        push @terms, $t;
    }
    return @terms;
}
