#!/usr/bin/env perl
use strict;
use warnings;
use Path::Class qw/file/;

use LWP::UserAgent;
use HTML::ExtractContent;
use Lingua::JA::TFIDF;
use Cache::File;

my $dat = shift;
my $inurl = shift  or die "usage: %0 <urlfile> url";
my $fh  = file($dat)->openr;

my %doc;
my $cache = Cache::File->new(
    cache_root => '/tmp/webdb_cache',
);

while (defined(my $line = $fh->getline)) {
    chomp $line;
    my ($doc_id, $url, $title) = split ',', $line;

    # 1度作ったベクトルはキャッシュ
    my $vec;
    if (not $vec = $cache->thaw($url)) {
        $vec = text2vec(url2text($url));
        $cache->freeze($url => $vec);
    }

    $doc{$doc_id} = {
        url   => $url,
        title => $title,
        vec   => $vec,
    };
}

# my $input_vec = text2vec(url2text($inurl));
my $input_vec = text2vec('サッカー日本代表');

my %sim;
for my $doc_id (keys %doc) {
    $sim{$doc_id} = cos_sim($input_vec, $doc{$doc_id}->{vec});
}

for my $doc_id (sort {$sim{$b} <=> $sim{$a}} keys %sim) {
    printf "%2d: %.2f %s\n",
        $doc_id,
        $sim{$doc_id},
        $doc{$doc_id}->{title};
}

sub parse_tags {
    my $vec = {};
    for (@_) {
        my ($tag, $occ) = split ':';
        $vec->{$tag} += $occ;
    }
    return $vec;
}

sub cos_sim {
    my ($v1, $v2) = @_;
    return dot_product($v1, $v2)
        / (vec_abs($v1) * vec_abs($v2));
}

sub dot_product {
    my ($v1, $v2) = @_;
    my $v = 0;
    for (keys %$v1) {
        if ($v1->{$_} and $v2->{$_}) {
            $v += $v1->{$_} * $v2->{$_};
        }
    }
    return $v;
}

sub vec_abs {
    my $vec = shift;
    my $v = 0;
    for (values %$vec) {
        $v += $_ * $_;
    }
    return sqrt $v;
}

sub url2text {
    my $url = shift;

    my $ua  = LWP::UserAgent->new;
    my $res = $ua->get($url);

    if ($res->is_error) {
        die $res->status_line;
    }

    my $html = $res->decoded_content;
    my $ex = HTML::ExtractContent->new;
    $ex->extract($html);
    return $ex->as_text;
}

sub text2vec {
    my $text = shift;
    my $tfidf = Lingua::JA::TFIDF->new;
    my $result = $tfidf->tfidf($text);
    my %vec = map { %$_ } @{$result->list};
    return \%vec;
}
